@@ -3,8 +3,6 @@
 
 class MainWindow;
 
-#ifdef MINIZINC_IDE_HAVE_WEBENGINE
-
 #include <QWebEnginePage>
 #include <QWebChannel>
 
@@ -18,6 +16,7 @@ public:
     MiniZincIDEJS(HTMLPage* p);
 public slots:
     void selectSolution(int n);
+    void solve(const QString& data);
 };
 
 class HTMLPage : public QWebEnginePage
@@ -27,48 +26,23 @@ protected:
     MainWindow* _mw;
     QWebChannel* _webChannel;
     MiniZincIDEJS* _mznide;
+    int _htmlWindowIdentifier;
     QStringList json;
     bool loadFinished;
     void runJs(QString js);
 public:
-    explicit HTMLPage(MainWindow* mw, QWidget *parent = 0);
+    explicit HTMLPage(MainWindow* mw, int htmlWindowIdentifier, QWidget *parent = 0);
     virtual void javaScriptConsoleMessage(JavaScriptConsoleMessageLevel level, const QString &message, int lineNumber, const QString &sourceID);
     void addSolution(const QString& json);
+    void initJSON(const QString& json);
     void showSolution(int n);
     void finish(qint64 runtime);
 public slots:
     void selectSolution(int n);
+    void solve(const QString& data);
 
 private slots:
     void pageLoadFinished(bool ok);
 };
 
-#else
-
-#include <QWebPage>
-
-class HTMLPage : public QWebPage
-{
-    Q_OBJECT
-protected:
-    MainWindow* _mw;
-    QStringList json;
-    bool loadFinished;
-    void runJs(QString js);
-public:
-    explicit HTMLPage(MainWindow* mw, QWidget *parent = 0);
-    virtual void javaScriptConsoleMessage(const QString &message, int lineNumber, const QString &sourceID);
-    void addSolution(const QString& json);
-    void showSolution(int n);
-    void finish(qint64 runtime);
-public slots:
-    void selectSolution(int n);
-
-private slots:
-    void pageLoadFinished(bool ok);
-    void jsCleared(void);
-};
-
-#endif
-
 #endif // HTMLPAGE_H
