@@ -32,14 +32,18 @@
 #include <QFileSystemWatcher>
 #include <QNetworkAccessManager>
 #include <QSortFilterProxyModel>
+#include <QToolButton>
+#include <QProgressBar>
+#include <QComboBox>
+#include <QVersionNumber>
 
 #include "codeeditor.h"
 #include "solverdialog.h"
-#include "help.h"
 #include "paramdialog.h"
 #include "project.h"
 #include "htmlwindow.h"
 #include "moocsubmission.h"
+#include "solverconfiguration.h"
 
 namespace Ui {
 class MainWindow;
@@ -84,7 +88,6 @@ public:
     IDEStatistics stats;
 
     MainWindow* lastDefaultProject;
-    Help* helpWindow;
     QMainWindow* cheatSheet;
 
     QNetworkAccessManager* networkManager;
@@ -92,6 +95,12 @@ public:
 
 #ifdef Q_OS_MAC
     QMenuBar* defaultMenuBar;
+    QMenu* recentFilesMenu;
+    QMenu* recentProjectsMenu;
+public slots:
+    void recentFileMenuAction(QAction*);
+    void recentProjectMenuAction(QAction*);
+public:
 #endif
 
     QFileSystemWatcher fsWatch;
@@ -108,12 +117,14 @@ public:
     QString getLastPath(void);
     void setLastPath(const QString& path);
     void setEditorFont(QFont font);
+    void addRecentFile(const QString& file);
+    void addRecentProject(const QString& file);
 protected:
     bool event(QEvent *);
 protected slots:
     void versionCheckFinished(void);
     void newProject(void);
-    void openFile(void);
+    void openFile(const QString& filename = QString(""));
     void fileModified(const QString&);
     void fileModifiedTimeout(void);
     void handleFocusChange(QWidget*,QWidget*);
@@ -133,14 +144,14 @@ public:
 
 private:
     void init(const QString& project);
-
+    void compileOrRun();
 signals:
     /// emitted when compilation and running of a model has finished
     void finished();
 
 public slots:
 
-    void openFile(const QString &path = QString(), bool openAsModified=false);
+    void openFile(const QString &path = QString(), bool openAsModified=false, bool focus=true);
     void on_actionStop_triggered();
 
 private slots:
@@ -159,7 +170,12 @@ private slots:
     void checkArgsOutput();
     void checkArgsFinished(int exitcode, QProcess::ExitStatus exitstatus);
 
+    void checkModelStarted();
+    void checkModelFinished(int exitcode, QProcess::ExitStatus exitstatus);
+    void checkModelError(QProcess::ProcessError);
+
     void readOutput();
+    void readRrofileOutput();
 
     void procFinished(int, bool showTime=true);
 
@@ -178,6 +194,8 @@ private slots:
 
     void openCompiledFzn(int);
 
+    void profileCompiledFzn(int);
+
     void runCompiledFzn(int,QProcess::ExitStatus);
 
     void on_actionSave_as_triggered();
@@ -240,6 +258,8 @@ private slots:
 
     void on_actionHide_tool_bar_triggered();
 
+    void on_actionToggle_profiler_info_triggered();
+
     void on_actionShow_project_explorer_triggered();
 
     void activateFileInProject(const QModelIndex&);
@@ -266,16 +286,15 @@ private slots:
 
     void on_conf_timeLimit_valueChanged(int arg1);
 
-    void on_conf_solver_activated(const QString &arg1);
-
     void onClipboardChanged();
 
-    void on_conf_data_file_activated(const QString &arg1);
+    void editor_cursor_position_changed();
 
     void showWindowMenu(void);
     void windowMenuSelected(QAction*);
-    void closeHTMLWindow(void);
+    void closeHTMLWindow(int);
     void on_actionCheat_Sheet_triggered();
+    void check_code();
 
     void on_actionDark_mode_toggled(bool arg1);
 
@@ -283,23 +302,75 @@ private slots:
 
     void on_defaultBehaviourButton_toggled(bool checked);
 
+    void on_actionEditSolverConfig_triggered();
+
+    void on_conf_solver_conf_currentIndexChanged(int index);
+
+    void on_solverConfigurationSelected(QAction*);
+
+    void on_cloneSolverConfButton_clicked();
+    void on_deleteSolverConfButton_clicked();
+    void on_renameSolverConfButton_clicked();
+
+    void on_solverConfNameEdit_returnPressed();
+    void on_solverConfNameEdit_escPressed();
+
+    void on_conf_default_toggled(bool checked);
+    void on_conf_dock_widget_visibilityChanged(bool visible);
+
+    void on_conf_check_solutions_toggled(bool checked);
+
+    void on_b_next_clicked();
+
+    void on_b_prev_clicked();
+
+    void on_b_replacefind_clicked();
+
+    void on_b_replace_clicked();
+
+    void on_b_replaceall_clicked();
+
+    void on_closeFindWidget_clicked();
+
+    void on_find_textEdited(const QString &arg1);
+
+    void on_extraOptionsBox_toggled(bool arg1);
+
+    void on_actionProfile_compilation_triggered();
+
 protected:
     virtual void closeEvent(QCloseEvent*);
     virtual void dragEnterEvent(QDragEnterEvent *);
     virtual void dropEvent(QDropEvent *);
+#ifdef Q_OS_MAC
+    virtual void paintEvent(QPaintEvent *);
+#endif
     bool eventFilter(QObject *, QEvent *);
     void openJSONViewer(void);
     void finishJSONViewer(void);
-    void compileAndRun(const QString& modelPath, const QString& additionalCmdlineParams, const QString& additionalDataFile);
+
+    SolverConfiguration* getCurrentSolverConfig(void);
+    Solver* getCurrentSolver(void);
+
+    void compileAndRun(const QString& modelPath, const QString& additionalCmdlineParams, const QStringList& additionalDataFiles);
 public:
     bool runWithOutput(const QString& modelFile, const QString& dataFile, int timeout, QTextStream& outstream);
+    void resolve(int htmlWindowIdentifier, const QString &data);
+    QString currentSolver(void) const;
+    QString currentSolverConfigName(void);
+    int addHtmlWindow(HTMLWindow* w);
 private:
     Ui::MainWindow *ui;
     CodeEditor* curEditor;
     QString curFilePath;
-    HTMLWindow* curHtmlWindow;
+    int curHtmlWindow;
+    QVector<HTMLWindow*> htmlWindows;
+    QVector<QString> htmlWindowModels;
     MznProcess* process;
+    MznProcess* check_process;
     QString processName;
+    QString curModelFilepath;
+    CodeEditor* curCheckEditor;
     MznProcess* outputProcess;
     bool processWasStopped;
     int solutionCount;
@@ -308,31 +379,48 @@ private:
     QVector<QString> hiddenSolutions;
     int curJSONHandler;
     bool inJSONHandler;
+    bool isJSONinitHandler;
+    QStringList htmlBuffer;
+    bool inHTMLHandler {false};
     bool hadNonJSONOutput;
     QVector<QStringList> JSONOutput;
     QTimer* timer;
     QTimer* solverTimeout;
+    QTimer* checkTimer;
     int time;
     QElapsedTimer elapsedTime;
+    QProgressBar* progressBar;
     QLabel* statusLabel;
+    QLabel* statusLineColLabel;
     QFont editorFont;
     bool darkMode;
     QVector<Solver> solvers;
-    QString defaultSolver;
+    QString userSolverConfigDir;
+    QString userConfigFile;
+    QString mznStdlibDir;
+    int defaultSolverIdx;
     QString mznDistribPath;
     QString getMznDistribPath(void) const;
     QString currentFznTarget;
+    QString currentPathsTarget;
+    bool isOptimisation;
+    QStringList currentAdditionalDataFiles;
     bool runSolns2Out;
     QTemporaryDir* tmpDir;
     QVector<QTemporaryDir*> cleanupTmpDirs;
     QVector<MznProcess*> cleanupProcesses;
-    FindDialog* findDialog;
+    QTextCursor incrementalFindCursor;
     QString projectPath;
     bool saveBeforeRunning;
     QString compileErrors;
+    QString checkArgsStdout;
     ParamDialog* paramDialog;
-    bool compileOnly;
-    QString mzn2fzn_executable;
+    enum CompileMode { CM_RUN, CM_COMPILE, CM_PROFILE };
+    CompileMode compileMode;
+    bool profileInfoVisible;
+    int runTimeout;
+    QString minizinc_executable;
+    QVersionNumber minizinc_version;
     Project project;
     QSortFilterProxyModel* projectSort;
     QMenu* projectContextMenu;
@@ -344,6 +432,7 @@ private:
     QString projectSelectedFile;
     QModelIndex projectSelectedIndex;
     int newFileCounter;
+    QComboBox* solverConfCombo;
     QAction* fakeRunAction;
     QAction* fakeStopAction;
     QAction* fakeCompileAction;
@@ -352,8 +441,16 @@ private:
     MOOCSubmission* moocSubmission;
     bool processRunning;
 
-    void createEditor(const QString& path, bool openAsModified, bool isNewFile, bool readOnly=false);
-    QStringList parseConf(bool compileOnly, bool useDataFile);
+    QToolButton* runButton;
+    QVector<SolverConfiguration> projectSolverConfigs;
+    QVector<SolverConfiguration> builtinSolverConfigs;
+    QVector<QPair<SolverFlag,QWidget*>> extraSolverFlags;
+    int currentSolverConfig;
+    bool renamingSolverConf;
+
+    void createEditor(const QString& path, bool openAsModified, bool isNewFile, bool readOnly=false, bool focus=true);
+    enum ConfMode { CONF_CHECKARGS, CONF_COMPILE, CONF_RUN };
+    QStringList parseConf(const ConfMode& confMode, const QString& modelFile, bool isOptimisation);
     void saveFile(CodeEditor* ce, const QString& filepath);
     void saveProject(const QString& filepath);
     void loadProject(const QString& filepath);
@@ -361,12 +458,16 @@ private:
     void setLastPath(const QString& s);
     QString getLastPath(void);
     QString setElapsedTime();
-    void setupDznMenu();
     void checkMznPath();
     void updateRecentProjects(const QString& p);
     void updateRecentFiles(const QString& p);
     void addFileToProject(bool dznOnly);
     void updateUiProcessRunning(bool pr);
+    void highlightPath(QString& path, int index);
+    QVector<CodeEditor*> collectCodeEditors(QVector<QStringList>& locs);
+    void updateSolverConfigs(void);
+    void setCurrentSolverConfig(int idx);
+    void find(bool fwd, bool forceNoWrapAround=false);
 public:
     void addOutput(const QString& s, bool html=true);
     void openProject(const QString& fileName);
