@@ -22,7 +22,7 @@
 #include <QJsonDocument>
 #include <QJsonArray>
 
-Project::Project(Ui::MainWindow *ui0) : ui(ui0), _moocAssignment(NULL), _courseraAssignment(NULL)
+Project::Project(Ui::MainWindow *ui0) : ui(ui0), _moocAssignment(nullptr), _courseraAssignment(nullptr)
 {
     projectFile = new QStandardItem("Untitled Project");
     invisibleRootItem()->appendRow(projectFile);
@@ -70,7 +70,7 @@ QVariant Project::data(const QModelIndex &index, int role) const
 {
     if (role==Qt::UserRole) {
         QStandardItem* item = itemFromIndex(index);
-        if (item->parent()==NULL || item->parent()==invisibleRootItem()) {
+        if (item->parent()==nullptr || item->parent()==invisibleRootItem()) {
             if (item==projectFile) {
                 return "00 - project";
             }
@@ -113,7 +113,7 @@ void Project::addFile(QTreeView* treeView, QSortFilterProxyModel* sort, const QS
     bool isMOOC = false;
     if (fi.suffix()=="mzn") {
         curItem = mzn;
-    } else if (fi.suffix()=="dzn") {
+    } else if (fi.suffix()=="dzn" || fi.suffix()=="json") {
         curItem = dzn;
     } else if (fi.suffix()=="fzn") {
         return;
@@ -125,13 +125,13 @@ void Project::addFile(QTreeView* treeView, QSortFilterProxyModel* sort, const QS
 
     if (isMOOC) {
 
-        if (fi.baseName()=="_coursera" && _courseraAssignment != NULL) {
+        if (fi.baseName()=="_coursera" && _courseraAssignment != nullptr) {
             QMessageBox::warning(treeView,"MiniZinc IDE",
                                 "Cannot add second Coursera options file",
                                 QMessageBox::Ok);
             return;
         }
-        if (fi.baseName()=="_mooc" && _moocAssignment != NULL) {
+        if (fi.baseName()=="_mooc" && _moocAssignment != nullptr) {
             QMessageBox::warning(treeView,"MiniZinc IDE",
                                 "Cannot add second MOOC options file",
                                 QMessageBox::Ok);
@@ -161,13 +161,13 @@ void Project::addFile(QTreeView* treeView, QSortFilterProxyModel* sort, const QS
             QTextStream in(&jsonString);
             if (in.status() != QTextStream::Ok) {
                 delete moocA;
-                moocA = NULL;
+                moocA = nullptr;
                 goto coursera_done;
             }
             moocA->assignmentKey = in.readLine();
             if (in.status() != QTextStream::Ok) {
                 delete moocA;
-                moocA = NULL;
+                moocA = nullptr;
                 goto coursera_done;
             }
             moocA->name = in.readLine();
@@ -176,14 +176,14 @@ void Project::addFile(QTreeView* treeView, QSortFilterProxyModel* sort, const QS
             for (int i=0; i<nSolutions; i++) {
                 if (in.status() != QTextStream::Ok) {
                     delete moocA;
-                    moocA = NULL;
+                    moocA = nullptr;
                     goto coursera_done;
                 }
                 QString line = in.readLine();
                 QStringList tokens = line.split(", ");
                 if (tokens.size() < 5) {
                     delete moocA;
-                    moocA = NULL;
+                    moocA = nullptr;
                     goto coursera_done;
                 }
                 MOOCAssignmentItem item(tokens[0].trimmed(),tokens[1].trimmed(),tokens[2].trimmed(),
@@ -192,7 +192,7 @@ void Project::addFile(QTreeView* treeView, QSortFilterProxyModel* sort, const QS
             }
             if (in.status() != QTextStream::Ok) {
                 delete moocA;
-                moocA = NULL;
+                moocA = nullptr;
                 goto coursera_done;
             }
             nSolutions_s = in.readLine();
@@ -200,14 +200,14 @@ void Project::addFile(QTreeView* treeView, QSortFilterProxyModel* sort, const QS
             for (int i=0; i<nSolutions; i++) {
                 if (in.status() != QTextStream::Ok) {
                     delete moocA;
-                    moocA = NULL;
+                    moocA = nullptr;
                     goto coursera_done;
                 }
                 QString line = in.readLine();
                 QStringList tokens = line.split(", ");
                 if (tokens.size() < 3) {
                     delete moocA;
-                    moocA = NULL;
+                    moocA = nullptr;
                     goto coursera_done;
                 }
                 MOOCAssignmentItem item(tokens[0].trimmed(),tokens[1].trimmed(),tokens[2].trimmed());
@@ -259,7 +259,7 @@ void Project::addFile(QTreeView* treeView, QSortFilterProxyModel* sort, const QS
                                      "MOOC options file contains errors",
                                      QMessageBox::Ok);
                 delete moocA;
-                moocA = NULL;
+                moocA = nullptr;
             }
         }
 
@@ -287,7 +287,7 @@ coursera_done:
     treeView->expand(sort->mapFromSource(curItem->index()));
     curItem = curItem->child(0);
     int i=0;
-    while (curItem != NULL) {
+    while (curItem != nullptr) {
         if (curItem->text() == path.first()) {
             path.pop_front();
             treeView->expand(sort->mapFromSource(curItem->index()));
@@ -318,10 +318,10 @@ coursera_done:
 QString Project::fileAtIndex(const QModelIndex &index)
 {
     QStandardItem* item = itemFromIndex(index);
-    if (item==NULL || item->hasChildren())
+    if (item==nullptr || item->hasChildren())
         return "";
     QString fileName;
-    while (item != NULL && item->parent() != NULL && item->parent() != invisibleRootItem() ) {
+    while (item != nullptr && item->parent() != nullptr && item->parent() != invisibleRootItem() ) {
         if (fileName.isEmpty())
             fileName = item->text();
         else
@@ -352,7 +352,7 @@ QStringList Project::dataFiles(void) const
 {
     QStringList ret;
     for (QMap<QString,QPersistentModelIndex>::const_iterator it = _files.begin(); it != _files.end(); ++it) {
-        if (it.key().endsWith(".dzn"))
+        if (it.key().endsWith(".dzn") || it.key().endsWith(".json"))
             ret << it.key();
     }
     return ret;
@@ -370,7 +370,7 @@ void Project::removeFile(const QString &fileName)
     QModelIndex index = _files[fileName];
     _files.remove(fileName);
     QStandardItem* cur = itemFromIndex(index);
-    while (cur->parent() != NULL && cur->parent() != invisibleRootItem() && !cur->hasChildren()) {
+    while (cur->parent() != nullptr && cur->parent() != invisibleRootItem() && !cur->hasChildren()) {
         int row = cur->row();
         cur = cur->parent();
         cur->removeRow(row);
@@ -378,14 +378,14 @@ void Project::removeFile(const QString &fileName)
     QFileInfo fi(fileName);
     if (fi.fileName()=="_coursera") {
         delete _courseraAssignment;
-        _courseraAssignment = NULL;
-        if (_moocAssignment==NULL) {
+        _courseraAssignment = nullptr;
+        if (_moocAssignment==nullptr) {
             ui->actionSubmit_to_MOOC->setVisible(false);
         }
     } else if (fi.fileName()=="_mooc") {
         delete _moocAssignment;
-        _moocAssignment = NULL;
-        if (_courseraAssignment!=NULL) {
+        _moocAssignment = nullptr;
+        if (_courseraAssignment!=nullptr) {
             ui->actionSubmit_to_MOOC->setText("Submit to "+_courseraAssignment->moocName);
             ui->actionSubmit_to_MOOC->setIcon(QIcon(":/icons/images/coursera.png"));
         } else {
@@ -394,6 +394,11 @@ void Project::removeFile(const QString &fileName)
     }
 }
 
+bool Project::containsFile(const QString &fileName)
+{
+    return _files.contains(fileName);
+}
+
 void Project::setModified(bool flag, bool files)
 {
     if (!projectRoot.isEmpty()) {
@@ -404,25 +409,7 @@ void Project::setModified(bool flag, bool files)
                 _filesModified = _isModified;
             }
             if (!_isModified) {
-                currentDataFileIndex(currentDataFileIndex(),true);
-                haveExtraArgs(haveExtraArgs(),true);
-                extraArgs(extraArgs(),true);
-                mzn2fznVerbose(mzn2fznVerbose(),true);
-                mzn2fznPrintStats(mzn2fznPrintStats(),true);
-                mzn2fznOptimize(mzn2fznOptimize(),true);
-                currentSolver(currentSolver(),true);
-                n_solutions(n_solutions(),true);
-                n_compress_solutions(n_compress_solutions(),true);
-                printAll(printAll(),true);
-                defaultBehaviour(defaultBehaviour(),true);
-                printStats(printStats(),true);
-                haveSolverFlags(haveSolverFlags(),true);
-                solverFlags(solverFlags(),true);
-                n_threads(n_threads(),true);
-                haveSeed(haveSeed(),true);
-                seed(seed(),true);
-                timeLimit(timeLimit(),true);
-                solverVerbose(solverVerbose(),true);
+                solverConfigs(solverConfigs(),true);
             }
         }
     }
@@ -446,94 +433,8 @@ bool Project::setData(const QModelIndex& index, const QVariant& value, int role)
     }
 }
 
-bool Project::haveExtraArgs(void) const
-{
-    return ui->conf_have_cmd_params->isChecked();
-}
-QString Project::extraArgs(void) const
-{
-    return ui->conf_cmd_params->text();
-}
-bool Project::haveExtraMzn2FznArgs(void) const
-{
-    return ui->conf_have_mzn2fzn_params->isChecked();
-}
-QString Project::extraMzn2FznArgs(void) const
-{
-    return ui->conf_mzn2fzn_params->text();
-}
-bool Project::autoClearOutput(void) const
-{
-    return ui->autoclear_output->isChecked();
-}
-bool Project::mzn2fznVerbose(void) const
-{
-    return ui->conf_verbose->isChecked();
-}
-
-bool Project::mzn2fznPrintStats() const
-{
-    return ui->conf_flatten_stats->isChecked();
-}
-bool Project::mzn2fznOptimize(void) const
-{
-    return ui->conf_optimize->isChecked();
-}
-QString Project::currentSolver(void) const
-{
-    return ui->conf_solver->currentText();
-}
-int Project::n_solutions(void) const
-{
-    return ui->conf_nsol->value();
-}
-int Project::n_compress_solutions(void) const
-{
-    return ui->conf_compressSolutionLimit->value();
-}
-bool Project::printAll(void) const
-{
-    return ui->conf_printall->isChecked();
-}
-bool Project::defaultBehaviour(void) const
-{
-    return ui->defaultBehaviourButton->isChecked();
-}
-bool Project::printStats(void) const
-{
-    return ui->conf_stats->isChecked();
-}
-bool Project::printTiming(void) const
-{
-    return ui->conf_solver_timing->isChecked();
-}
-bool Project::haveSolverFlags(void) const
-{
-    return ui->conf_have_solverFlags->isChecked();
-}
-QString Project::solverFlags(void) const
-{
-    return ui->conf_solverFlags->text();
-}
-int Project::n_threads(void) const
-{
-    return ui->conf_nthreads->value();
-}
-bool Project::haveSeed(void) const
-{
-    return ui->conf_have_seed->isChecked();
-}
-QString Project::seed(void) const
-{
-    return ui->conf_seed->text();
-}
-int Project::timeLimit(void) const
-{
-    return ui->conf_timeLimit->value();
-}
-bool Project::solverVerbose(void) const
-{
-    return ui->conf_solver_verbose->isChecked();
+const QVector<SolverConfiguration>& Project::solverConfigs(void) const {
+    return _solverConfigs;
 }
 
 bool Project::isUndefined() const
@@ -541,330 +442,22 @@ bool Project::isUndefined() const
     return projectRoot.isEmpty();
 }
 
-void Project::currentDataFileIndex(int i, bool init)
+void Project::solverConfigs(const QVector<SolverConfiguration> &sc, bool init)
 {
     if (init) {
-        _currentDatafileIndex = i;
-        ui->conf_data_file->setCurrentIndex(i);
+        _solverConfigs = sc;
     } else {
-        if (i < ui->conf_data_file->count()-1)
-            checkModified();
+        if (sc.size() != _solverConfigs.size()) {
+            setModified(true);
+        } else {
+            bool modified = false;
+            for (int i=0; i<sc.size(); i++) {
+                if (!(sc[i]==_solverConfigs[i])) {
+                    modified = true;
+                    _solverConfigs[i] = sc[i];
+                }
+            }
+            setModified(modified);
+        }
     }
 }
-
-void Project::haveExtraArgs(bool b, bool init)
-{
-    if (init) {
-        _haveExtraArgs = b;
-        ui->conf_have_cmd_params->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::extraArgs(const QString& a, bool init)
-{
-    if (init) {
-        _extraArgs = a;
-        ui->conf_cmd_params->setText(a);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::haveExtraMzn2FznArgs(bool b, bool init)
-{
-    if (init) {
-        _haveExtraMzn2FznArgs = b;
-        ui->conf_have_mzn2fzn_params->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::extraMzn2FznArgs(const QString& a, bool init)
-{
-    if (init) {
-        _extraMzn2FznArgs = a;
-        ui->conf_mzn2fzn_params->setText(a);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::autoClearOutput(bool b, bool init)
-{
-    if (init) {
-        _autoclear_output= b;
-        ui->autoclear_output->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::mzn2fznVerbose(bool b, bool init)
-{
-    if (init) {
-        _mzn2fzn_verbose= b;
-        ui->conf_verbose->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::mzn2fznPrintStats(bool b, bool init)
-{
-    if (init) {
-        _mzn2fzn_printStats = b;
-        ui->conf_flatten_stats->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::mzn2fznOptimize(bool b, bool init)
-{
-    if (init) {
-        _mzn2fzn_optimize = b;
-        ui->conf_optimize->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::currentSolver(const QString& s, bool init)
-{
-    if (init) {
-        _currentSolver = s;
-        ui->conf_solver->setCurrentText(s);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::n_solutions(int n, bool init)
-{
-    if (init) {
-        _n_solutions = n;
-        ui->conf_nsol->setValue(n);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::n_compress_solutions(int n, bool init)
-{
-    if (init) {
-        _compressSolutionLimit = n;
-        ui->conf_compressSolutionLimit->setValue(n);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::printAll(bool b, bool init)
-{
-    if (init) {
-        _printAll = b;
-        ui->conf_printall->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::defaultBehaviour(bool b, bool init)
-{
-    if (init) {
-        _defaultBehaviour = b;
-        ui->defaultBehaviourButton->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::printStats(bool b, bool init)
-{
-    if (init) {
-        _printStats = b;
-        ui->conf_stats->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::printTiming(bool b, bool init)
-{
-    if (init) {
-        _printTiming = b;
-        ui->conf_solver_timing->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::haveSolverFlags(bool b, bool init)
-{
-    if (init) {
-        _haveSolverFlags = b;
-        ui->conf_have_solverFlags->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::solverFlags(const QString& s, bool init)
-{
-    if (init) {
-        _solverFlags = s;
-        ui->conf_solverFlags->setText(s);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::n_threads(int n, bool init)
-{
-    if (init) {
-        _n_threads = n;
-        ui->conf_nthreads->setValue(n);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::haveSeed(bool b, bool init)
-{
-    if (init) {
-        _haveSeed = b;
-        ui->conf_have_seed->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::seed(const QString& s, bool init)
-{
-    if (init) {
-        _seed = s;
-        ui->conf_seed->setText(s);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::timeLimit(int n, bool init)
-{
-    if (init) {
-        _timeLimit = n;
-        ui->conf_timeLimit->setValue(n);
-    } else {
-        checkModified();
-    }
-}
-
-void Project::solverVerbose(bool b, bool init)
-{
-    if (init) {
-        _solverVerbose = b;
-        ui->conf_solver_verbose->setChecked(b);
-    } else {
-        checkModified();
-    }
-}
-
-int Project::currentDataFileIndex(void) const
-{
-    return ui->conf_data_file->currentIndex();
-}
-
-QString Project::currentDataFile(void) const
-{
-    return ui->conf_data_file->currentText();
-}
-
-void Project::checkModified()
-{
-    if (projectRoot.isEmpty() || _filesModified)
-        return;
-    if (currentDataFileIndex() != _currentDatafileIndex) {
-        setModified(true);
-        return;
-    }
-    if (haveExtraArgs() != _haveExtraArgs) {
-        setModified(true);
-        return;
-    }
-    if (extraArgs() != _extraArgs) {
-        setModified(true);
-        return;
-    }
-    if (mzn2fznVerbose() != _mzn2fzn_verbose) {
-        setModified(true);
-        return;
-    }
-    if (mzn2fznPrintStats() != _mzn2fzn_printStats) {
-        setModified(true);
-        return;
-    }
-    if (mzn2fznOptimize() != _mzn2fzn_optimize) {
-        setModified(true);
-        return;
-    }
-    if (currentSolver() != _currentSolver) {
-        setModified(true);
-        return;
-    }
-    if (n_solutions() != _n_solutions) {
-        setModified(true);
-        return;
-    }
-    if (n_compress_solutions() != _compressSolutionLimit) {
-        setModified(true);
-        return;
-    }
-    if (printAll() != _printAll) {
-        setModified(true);
-        return;
-    }
-    if (defaultBehaviour() != _defaultBehaviour) {
-        setModified(true);
-        return;
-    }
-    if (printStats() != _printStats) {
-        setModified(true);
-        return;
-    }
-    if (printTiming() != _printTiming) {
-        setModified(true);
-        return;
-    }
-    if (haveSolverFlags() != _haveSolverFlags) {
-        setModified(true);
-        return;
-    }
-    if (solverFlags() != _solverFlags) {
-        setModified(true);
-        return;
-    }
-    if (n_threads() != _n_threads) {
-        setModified(true);
-        return;
-    }
-    if (haveSeed() != _haveSeed) {
-        setModified(true);
-        return;
-    }
-    if (seed() != _seed) {
-        setModified(true);
-        return;
-    }
-    if (timeLimit() != _timeLimit) {
-        setModified(true);
-        return;
-    }
-    if (solverVerbose() != _solverVerbose) {
-        setModified(true);
-        return;
-    }
-    setModified(false);
-}
